/*
  Rhino test framework

   Copyright (C) 2011, 2012, 2013, 2014  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.RhinoTests;

import javax.script.Compilable;
import javax.script.CompiledScript;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;

/**
 * This test case check the behavior of CompiledScript abstract class and it's
 * descendants.
 * 
 * @author Pavel Tisnovsky
 * 
 */
public class CompilableTest extends BaseRhinoTest {

    /**
     * Instance of ScriptEngineManager which is used by all tests in this test
     * case.
     */
    ScriptEngineManager engineManager;

    /**
     * Instance of ScriptEngine which is used by all tests in this test case.
     */
    ScriptEngine scriptEngine;

    @Override
    protected void setUp(String[] args) {
        // setup attribute used by tests
        this.engineManager = new ScriptEngineManager();
        this.scriptEngine = this.engineManager.getEngineByName("JavaScript");
    }

    @Override
    protected void tearDown() {
        // this block could be empty
        return;
    }

    /**
     * Helper method which tries to retrieve an instance of class which
     * implements CompiledScript interface for a given script.
     * 
     * @param scriptText
     *            script source code
     * @return instance of CompiledScript class
     * @throws AssertionError
     *             when CompilingEngine cannot be retrieved
     * @throws ScriptException
     *             thrown when script cannot be compiled
     */
    private CompiledScript getCompiledScript(String scriptText) throws AssertionError, ScriptException {
        // check if retyping could be done
        assertTrue(this.scriptEngine instanceof Compilable, "ScriptEngine does not implement Compilable");
        // scriptEngine should be also retyped to Compilable, at least in case of JavaScript.
        Compilable compilingEngine = (Compilable) this.scriptEngine;
        // should not happen, but...
        assertNotNull(compilingEngine, "cannot get compiling engine");
        // try to compile given script
        return compileScript(scriptText, compilingEngine);
    }

    /**
     * Helper method which tries to compile given JavaScript.
     *
     * @param scriptText script source code
     * @param compilingEngine instance of class which implements Compilable interface
     * @return compiled script
     * @throws ScriptException
     * @throws AssertionError
     */
    private CompiledScript compileScript(String scriptText, Compilable compilingEngine) throws ScriptException, AssertionError {
        CompiledScript script = compilingEngine.compile(scriptText);
        assertNotNull(script, "cannot compile script");
        return script;
    }

    /**
     * Test if it is possible to compile script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileEmptyScriptStoredInString1() throws ScriptException {
        Compilable compilingEngine = (Compilable)this.scriptEngine;
        assertNotNull(compilingEngine, "cannot get compiling engine");
        if (compilingEngine != null) {
            CompiledScript script = compilingEngine.compile(JavaScriptSnippets.EMPTY_SCRIPT_1);
            assertNotNull(script, "cannot compile script");
        }
    }

    /**
     * Test if it is possible to compile script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileEmptyScriptStoredInString2() throws ScriptException {
        Compilable compilingEngine = (Compilable)this.scriptEngine;
        assertNotNull(compilingEngine, "cannot get compiling engine");
        if (compilingEngine != null) {
            CompiledScript script = compilingEngine.compile(JavaScriptSnippets.EMPTY_SCRIPT_2);
            assertNotNull(script, "cannot compile script");
        }
    }

    /**
     * Test if it is possible to compile script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileEmptyScriptStoredInString3() throws ScriptException {
        Compilable compilingEngine = (Compilable)this.scriptEngine;
        assertNotNull(compilingEngine, "cannot get compiling engine");
        if (compilingEngine != null) {
            CompiledScript script = compilingEngine.compile(JavaScriptSnippets.EMPTY_SCRIPT_3);
            assertNotNull(script, "cannot compile script");
        }
    }

    /**
     * Test if it is possible to compile script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileEmptyScriptStoredInString4() throws ScriptException {
        Compilable compilingEngine = (Compilable)this.scriptEngine;
        assertNotNull(compilingEngine, "cannot get compiling engine");
        if (compilingEngine != null) {
            CompiledScript script = compilingEngine.compile(JavaScriptSnippets.EMPTY_SCRIPT_4);
            assertNotNull(script, "cannot compile script");
        }
    }

    /**
     * Test if it is possible to compile script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileEmptyScriptStoredInString5() throws ScriptException {
        Compilable compilingEngine = (Compilable)this.scriptEngine;
        assertNotNull(compilingEngine, "cannot get compiling engine");
        if (compilingEngine != null) {
            CompiledScript script = compilingEngine.compile(JavaScriptSnippets.EMPTY_SCRIPT_5);
            assertNotNull(script, "cannot compile script");
        }
    }

    /**
     * Test if it is possible to compile script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileEmptyScriptStoredInString6() throws ScriptException {
        Compilable compilingEngine = (Compilable)this.scriptEngine;
        assertNotNull(compilingEngine, "cannot get compiling engine");
        if (compilingEngine != null) {
            CompiledScript script = compilingEngine.compile(JavaScriptSnippets.EMPTY_SCRIPT_6);
            assertNotNull(script, "cannot compile script");
        }
    }

    /**
     * Test if it is possible to compile script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileEmptyScriptStoredInString7() throws ScriptException {
        Compilable compilingEngine = (Compilable)this.scriptEngine;
        assertNotNull(compilingEngine, "cannot get compiling engine");
        if (compilingEngine != null) {
            CompiledScript script = compilingEngine.compile(JavaScriptSnippets.EMPTY_SCRIPT_7);
            assertNotNull(script, "cannot compile script");
        }
    }

    /**
     * Test if it is possible to compile script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileEmptyScriptStoredInString8() throws ScriptException {
        Compilable compilingEngine = (Compilable)this.scriptEngine;
        assertNotNull(compilingEngine, "cannot get compiling engine");
        if (compilingEngine != null) {
            CompiledScript script = compilingEngine.compile(JavaScriptSnippets.EMPTY_SCRIPT_8);
            assertNotNull(script, "cannot compile script");
        }
    }

    /**
     * Test if it is possible to compile script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileEmptyScriptStoredInString9() throws ScriptException {
        Compilable compilingEngine = (Compilable)this.scriptEngine;
        assertNotNull(compilingEngine, "cannot get compiling engine");
        if (compilingEngine != null) {
            CompiledScript script = compilingEngine.compile(JavaScriptSnippets.EMPTY_SCRIPT_9);
            assertNotNull(script, "cannot compile script");
        }
    }

    /**
     * Test if it is possible to compile script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileEmptyScriptStoredInString10() throws ScriptException {
        Compilable compilingEngine = (Compilable)this.scriptEngine;
        assertNotNull(compilingEngine, "cannot get compiling engine");
        if (compilingEngine != null) {
            CompiledScript script = compilingEngine.compile(JavaScriptSnippets.EMPTY_SCRIPT_10);
            assertNotNull(script, "cannot compile script");
        }
    }

    /**
     * Test if it is possible to compile script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileEmptyScriptStoredInString11() throws ScriptException {
        Compilable compilingEngine = (Compilable)this.scriptEngine;
        assertNotNull(compilingEngine, "cannot get compiling engine");
        if (compilingEngine != null) {
            CompiledScript script = compilingEngine.compile(JavaScriptSnippets.EMPTY_SCRIPT_11);
            assertNotNull(script, "cannot compile script");
        }
    }

    /**
     * Test if it is possible to compile script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileEmptyScriptStoredInString12() throws ScriptException {
        Compilable compilingEngine = (Compilable)this.scriptEngine;
        assertNotNull(compilingEngine, "cannot get compiling engine");
        if (compilingEngine != null) {
            CompiledScript script = compilingEngine.compile(JavaScriptSnippets.EMPTY_SCRIPT_12);
            assertNotNull(script, "cannot compile script");
        }
    }

    /**
     * Test if it is possible to compile script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileEmptyScriptStoredInString13() throws ScriptException {
        Compilable compilingEngine = (Compilable)this.scriptEngine;
        assertNotNull(compilingEngine, "cannot get compiling engine");
        if (compilingEngine != null) {
            CompiledScript script = compilingEngine.compile(JavaScriptSnippets.EMPTY_SCRIPT_13);
            assertNotNull(script, "cannot compile script");
        }
    }

    /**
     * Test if it is possible to compile script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileEmptyScriptStoredInString14() throws ScriptException {
        Compilable compilingEngine = (Compilable)this.scriptEngine;
        assertNotNull(compilingEngine, "cannot get compiling engine");
        if (compilingEngine != null) {
            CompiledScript script = compilingEngine.compile(JavaScriptSnippets.EMPTY_SCRIPT_14);
            assertNotNull(script, "cannot compile script");
        }
    }

    /**
     * Test if it is possible to compile and then run script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileAndRunSimpleScriptStoredInString1() throws ScriptException {
        CompiledScript script = getCompiledScript(JavaScriptSnippets.EMPTY_SCRIPT_1);
        Object result = script.eval();
        assertNull(result, "result should be null");
    }

    /**
     * Test if it is possible to compile and then run script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileAndRunSimpleScriptStoredInString2() throws ScriptException {
        CompiledScript script = getCompiledScript(JavaScriptSnippets.EMPTY_SCRIPT_2);
        Object result = script.eval();
        assertNull(result, "result should be null");
    }

    /**
     * Test if it is possible to compile and then run script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileAndRunSimpleScriptStoredInString3() throws ScriptException {
        CompiledScript script = getCompiledScript(JavaScriptSnippets.EMPTY_SCRIPT_3);
        Object result = script.eval();
        assertNull(result, "result should be null");
    }

    /**
     * Test if it is possible to compile and then run script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileAndRunSimpleScriptStoredInString4() throws ScriptException {
        CompiledScript script = getCompiledScript(JavaScriptSnippets.EMPTY_SCRIPT_4);
        Object result = script.eval();
        assertNull(result, "result should be null");
    }

    /**
     * Test if it is possible to compile and then run script from a string.
     * 
     * @throws ScriptException
     *             this exception is thrown when this test case failed.
     */
    protected void testCompileAndRunSimpleScriptStoredInString5() throws ScriptException {
        CompiledScript script = getCompiledScript(JavaScriptSnippets.EMPTY_SCRIPT_5);
        Object result = script.eval();
        assertNull(result, "result should be null");
    }

    /**
     * Entry point to this test case.
     *
     * @param args parameters passed from command line
     */
    public static void main(String[] args) {
        new CompilableTest().doTests(args);
    }

}
